@@ -30,8 +30,13 @@ LOG = utils.getProcessingLogger(__name__)
 VALID_STATES = {'enroll', 'manageable', 'inspecting', 'inspect wait',
                 'inspect failed'}
 
-# 1.19 is API version, which supports port.pxe_enabled
-DEFAULT_IRONIC_API_VERSION = '1.19'
+# 1.38 is the latest API version in the Queens release series, 10.1.0.
+# 1.46 is the latest API version in the Rocky release series, 11.1.0.
+# NOTE(mgoddard): This should be updated with each release to ensure that
+# inspector is able to use the latest ironic API. In particular, this version
+# is used when processing introspection rules, and is the default version used
+# by processing plugins.
+DEFAULT_IRONIC_API_VERSION = ['1.38', '1.46']
 
 IRONIC_SESSION = None
 
@@ -54,18 +59,43 @@ def reset_ironic_session():
 
 
 def get_ipmi_address(node):
+    """Get the BMC address defined in node.driver_info dictionary
+
+    Possible names of BMC address value examined in order of list
+    ['ipmi_address'] + CONF.ipmi_address_fields. The value could
+    be an IP address or a hostname. DNS lookup performed for the
+    first non empty value.
+
+    The first valid BMC address value returned along with
+    it's v4 and v6 IP addresses.
+
+    :param node: Node object with defined driver_info dictionary
+    :return: tuple (ipmi_address, ipv4_address, ipv6_address)
+    """
+    none_address = None, None, None
     ipmi_fields = ['ipmi_address'] + CONF.ipmi_address_fields
     # NOTE(sambetts): IPMI Address is useless to us if bridging is enabled so
     # just ignore it and return None
     if node.driver_info.get("ipmi_bridging", "no") != "no":
-        return
+        return none_address
     for name in ipmi_fields:
         value = node.driver_info.get(name)
         if not value:
             continue
 
+        ipv4 = None
+        ipv6 = None
         try:
-            ip = socket.gethostbyname(value)
+            addrinfo = socket.getaddrinfo(value, None, 0, 0, socket.SOL_TCP)
+            for family, socket_type, proto, canon_name, sockaddr in addrinfo:
+                ip = sockaddr[0]
+                if netaddr.IPAddress(ip).is_loopback():
+                    LOG.warning('Ignoring loopback BMC address %s', ip,
+                                node_info=node)
+                elif family == socket.AF_INET:
+                    ipv4 = ip
+                elif family == socket.AF_INET6:
+                    ipv6 = ip
         except socket.gaierror:
             msg = _('Failed to resolve the hostname (%(value)s)'
                     ' for node %(uuid)s')
@@ -73,12 +103,8 @@ def get_ipmi_address(node):
                                      'uuid': node.uuid},
                               node_info=node)
 
-        if netaddr.IPAddress(ip).is_loopback():
-            LOG.warning('Ignoring loopback BMC address %s', ip,
-                        node_info=node)
-            ip = None
-
-        return ip
+        return (value, ipv4, ipv6) if ipv4 or ipv6 else none_address
+    return none_address
 
 
 def get_client(token=None,
@@ -97,6 +123,7 @@ def get_client(token=None,
         IRONIC_SESSION = keystone.get_session('ironic')
 
     args = {
+        'session': IRONIC_SESSION,
         'os_ironic_api_version': api_version,
         'max_retries': CONF.ironic.max_retries,
         'retry_interval': CONF.ironic.retry_interval}
@@ -104,11 +131,8 @@ def get_client(token=None,
     adapter_opts = dict()
 
     # TODO(pas-ha) use service auth with incoming token
-    if CONF.ironic.auth_type != 'none':
-        if token is None:
-            args['session'] = IRONIC_SESSION
-        else:
-            args['token'] = token
+    if CONF.ironic.auth_type != 'none' and token is not None:
+        args['token'] = token
 
     # TODO(pas-ha): remove handling of deprecated options in Rocky
     if CONF.ironic.os_region and not CONF.ironic.region_name:
@@ -121,7 +145,11 @@ def get_client(token=None,
     adapter = keystone.get_adapter('ironic', session=IRONIC_SESSION,
                                    **adapter_opts)
     endpoint = adapter.get_endpoint()
-    return client.Client(1, endpoint, **args)
+    if not endpoint:
+        raise utils.Error(
+            _('Cannot find the bare metal endpoint either in Keystone or '
+              'in the configuration'), code=500)
+    return client.get_client(1, endpoint=endpoint, **args)
 
 
 def check_provision_state(node):
